/*++

Copyright (c) 2009
Lysann Kessler, Stephanie Platz, Thomas Klingbeil, Philipp Tessenow, Frank Schlegel;
Michael Haupt, Michael Perscheid, Robert Hirschfeld;
Software Architecture Group, Hasso-Plattner-Institut, University of Potsdam, Germany

This software is published under MIT license.
It uses phidget library by Phidgets Inc. (http://phidgets.com).
See ../../../../license.pdf for more information.

Module Name:

	PhidgetCompInput.c

Abstract:

	This file contains methods for a Phidget Component Input. 
	A component Input can belong to several phidgets e.g. the phidget Motor Control or
	the phidget InterfaceKit. Only if one of these phidgets is attached the component
	is useable.	Use	phidget_GetDeviceClass(pointerToPhidget) to call the specific
	phidget methods	in phidget21 API.

--*/

#include "PhidgetCompDigitalInput.h"

//////////////////////////////////////////////////////////////////////////
// methods

//
// Gets the number of digital inputs supported by the owning phidget 
// of this component. Use id to identify the owning phidget.
//
int phidgetCompDigitalInput_GetCount(PhidgetId id)
{
	int iCount = 0;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, iCount);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
		case PHIDCLASS_INTERFACEKIT:
			CPhidgetInterfaceKit_getInputCount ( ((PhidgetIfKit*) pPhidget)->handle, &iCount);
			break;
		case PHIDCLASS_MOTORCONTROL:
			CPhidgetMotorControl_getInputCount ( ((PhidgetMotCon*) pPhidget)->handle, &iCount);
			break;
			
		default:
			IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompDigitalInput_GetCount.", phidget_GetDeviceClass(pPhidget)));
	}
	
	return iCount;
}

//
// Gets the current state (on or off) of a component digital input with the
// corresponding index. Use id to identify the owning phidget.
//
BOOL phidgetCompDigitalInput_GetValue(PhidgetId id, int iIndex)
{
	BOOL bValue = FALSE;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, bValue);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
		case PHIDCLASS_INTERFACEKIT:
			CPhidgetInterfaceKit_getInputState ( ((PhidgetIfKit*) pPhidget)->handle, iIndex, &bValue);
			break;
		case PHIDCLASS_MOTORCONTROL:
			CPhidgetMotorControl_getInputState ( ((PhidgetMotCon*) pPhidget)->handle, iIndex, &bValue);
			break;
			
		default:
			IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompDigitalInput_GetValue.", phidget_GetDeviceClass(pPhidget)));
	}
	
	return bValue > 0;
}

/*++

Routine Description:

	This method sets a semaphore to be signaled when the phidget with the
	corresponding id has changed a components digital input state. 

Arguments:

	id - id of the Phidget to set the semaphore.

	iSemaphoreIndex - the semaphore the iSemaphoreInputChange is set to.

Return Value:

	TRUE if the operation succeeded, FALSE otherwise.

--*/
BOOL phidgetCompDigitalInput_SetSemaphoreIndex_StateChange(PhidgetId id, int iSemaphoreIndex)
{
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_INTERFACEKIT:
		((PhidgetIfKit*) pPhidget)->iSemaphoreInputChange = iSemaphoreIndex;
		IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "New semaphore for <IfKit-InputChange> is: %d, Phidget id = %i",
			((PhidgetIfKit*) pPhidget)->iSemaphoreInputChange, pPhidget->id));
		return TRUE;

	case PHIDCLASS_MOTORCONTROL:
		((PhidgetMotCon*) pPhidget)->iSemaphoreInputChange = iSemaphoreIndex;
		IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "New semaphore for <MotCon-InputChange> is: %d, Phidget id = %i",
			((PhidgetMotCon*) pPhidget)->iSemaphoreInputChange, pPhidget->id));
		return TRUE;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompDigitalInput_SetSemaphoreIndex_StateChange.", phidget_GetDeviceClass(pPhidget)));
		return FALSE;
	}
}
