/*++

Copyright (c) 2009
Lysann Kessler, Stephanie Platz, Thomas Klingbeil, Philipp Tessenow, Frank Schlegel;
Michael Haupt, Michael Perscheid, Robert Hirschfeld;
Software Architecture Group, Hasso-Plattner-Institut, University of Potsdam, Germany

This software is published under MIT license.
It uses phidget library by Phidgets Inc. (http://phidgets.com).
See ../../../../license.pdf for more information.

Module Name:

	PhidgetAccelerometer.c

Abstract:

	This file contains Phidget Accelerometer specific methods. A Phidget 
	Accelerometer has a component axis.	Both modules - PhidgetCompAxis.c
	and PhidgetAccelerometer.c - implement all methods supplied by the
	phidget21 API for an Accelerometer.

--*/

#include "PhidgetAccelerometer.h"

//////////////////////////////////////////////////////////////////////////
// forward declarations: callback functions

int CALLBACK phidgetAccel_accelerationChangeHandler(CPhidgetAccelerometerHandle accelHandle, void *pUserPtr, int iIndex, double dAcceleration);

//////////////////////////////////////////////////////////////////////////
// PhidgetAccel methods

/*++

Routine Description:

	This method creates an PhidgetAccel struct and allocates memory.
	To identify the phidgets type the deviceClass is set here.
	Additionally do common creational settings.

Arguments:

	None.

Return Value:

	Squeak VM identifier for the PhigdetId Object.

--*/
sqInt phidgetAccel_Create()
{
	// allocate memory for a Accelerometer structure
	PhidgetAccel *pPhidget = calloc(1, sizeof(PhidgetAccel));

	// initialise
	pPhidget->deviceClass = PHIDCLASS_ACCELEROMETER;
	phidgetAccel_ResetSemaphores(pPhidget);

	// get id and add to global list. return the id
	return phidget_Create((Phidget*)pPhidget);
}

//
//reset semaphores
//
void phidgetAccel_ResetSemaphores(PhidgetAccel *pPhidget)
{
	if(pPhidget != NULL)
	{
		//reset common semaphores
		phidget_ResetSemaphores((Phidget*)pPhidget);
		//reset specific semaphores
		pPhidget->iSemaphoreAccelerationChange = 0;
	}
	else
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Null pointer given in phidgetAccel_ResetSemaphores for pPhidget."));
}

/*++

Routine Description:

	This method is called while opening a phidget. The actuall specific
	phidget object will be created and the specific callbacks will be
	registered.

Arguments:

	pPhidget - pointer of the Phidget to open the hardware connection.

Return Value:

	TRUE if the operation succeeded, FALSE otherwise.

--*/
BOOL phidgetAccel_Open(PhidgetAccel* pPhidget)
{
	if(pPhidget == NULL)
	{
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Null pointer given in phidgetAccel_Open for pPhidget."));
		return FALSE;
	}

	//create the Accelerometer object
	pPhidget->handle = 0;
	CPhidgetAccelerometer_create(&(pPhidget->handle));

	//Registers the callbacks that will run if an acceleration occurs by more than the OnAccelerationChange trigger.
	//Requires the handle for the Phidget, the function that will be called, and an arbitrary pointer that will be supplied to the callback function (may be NULL).
	CPhidgetAccelerometer_set_OnAccelerationChange_Handler (pPhidget->handle, phidgetAccel_accelerationChangeHandler,(void *)pPhidget->id);

	// do phidget common open operations
	phidget_Open_Base((Phidget*)pPhidget);
	return TRUE;
}

//////////////////////////////////////////////////////////////////////////
// callback functions

/*++

Routine Description:

	This callback is called when an axis of a Phidget Accelerometer with 
	the id pUserPtr	is changed by more than the OnAccelerationChange
	trigger.
	It signals the semaphore iSemaphoreAccelerationChange.

Arguments:

	accelHandle - handle of the Accelerometer to signal the semaphore.

	pUserPtr - Callback function pointer.

	iIndex - index of the axis that generated the event
	
	dAcceleration - the axis read acceleration

Return Value:

	1 if the operation succeeded, 0 otherwise.

--*/
int CALLBACK phidgetAccel_accelerationChangeHandler(CPhidgetAccelerometerHandle accelHandle, void *pUserPtr, int iIndex, double dAcceleration)
{
	PhidgetId id = (PhidgetId) pUserPtr;
	PhidgetAccel* pPhidget = (PhidgetAccel*) phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, 1);

	IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "<Accel-AccelerationChange> id = %i, semaphore = %i, index = %i, value = %f",
		pPhidget->id, pPhidget->iSemaphoreAccelerationChange, iIndex, dAcceleration));

	if (pPhidget->iSemaphoreAccelerationChange > 0)
		interpreterProxy->signalSemaphoreWithIndex(pPhidget->iSemaphoreAccelerationChange);
	else
		IF_DBG(SLog(SLOG_LEVEL_WARNING, "No semaphore set for <Accel-AccelerationChange>, id = %i", pPhidget->id));
	return 0;
}
