/*++

Copyright (c) 2009
Lysann Kessler, Stephanie Platz, Thomas Klingbeil, Philipp Tessenow, Frank Schlegel;
Michael Haupt, Michael Perscheid, Robert Hirschfeld;
Software Architecture Group, Hasso-Plattner-Institut, University of Potsdam, Germany

This software is published under MIT license.
It uses phidget library by Phidgets Inc. (http://phidgets.com).
See ../../../../license.pdf for more information.

Module Name:

	PhidgetCompOutput.c

Abstract:

	This file contains methods for a Phidget Component Output. 
	A component Output can belong to several phidgets e.g. the phidget RFID or the
	phidget InterfaceKit. Only if one of these phidgets is attached the component
	is useable.	Use	phidget_GetDeviceClass(pointerToPhidget) to call the specific
	phidget methods	in phidget21 API.

--*/

#include "PhidgetCompDigitalOutput.h"

//////////////////////////////////////////////////////////////////////////
// methods

//
// Gets the number of digital outputs supported by the owning phidget 
// of this component. Use id to identify the owning phidget.
//
int phidgetCompDigitalOutput_GetCount(PhidgetId id)
{
	int iCount = 0;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, iCount);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
		case PHIDCLASS_INTERFACEKIT:
			CPhidgetInterfaceKit_getOutputCount ( ((PhidgetIfKit*) pPhidget)->handle, &iCount);
			break;
		case PHIDCLASS_RFID:
			CPhidgetRFID_getOutputCount ( ((PhidgetRFID*) pPhidget)->handle, &iCount);
			break;
			
		default:
			IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompDigitalOutput_GetCount.", phidget_GetDeviceClass(pPhidget)));
	}

	return iCount;
}

//
// Gets the current state (on or off) of a component digital output with the
// corresponding index. Use id to identify the owning phidget.
//
BOOL phidgetCompDigitalOutput_GetValue(PhidgetId id, int iIndex)
{
	BOOL bValue = FALSE;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, bValue);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
		case PHIDCLASS_INTERFACEKIT:
			CPhidgetInterfaceKit_getOutputState ( ((PhidgetIfKit*) pPhidget)->handle, iIndex, &bValue);
			break;
		case PHIDCLASS_RFID:
			CPhidgetRFID_getOutputState ( ((PhidgetRFID*) pPhidget)->handle, iIndex, &bValue);
			break;
			
		default:
			IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompDigitalOutput_GetValue.", phidget_GetDeviceClass(pPhidget)));
	}
	
	return bValue > 0;
}

//
// Sets the state (on or off) of a component digital input with the
// corresponding index. Use id to identify the owning phidget.
// Return TRUE if the setting succeeded, FALSE otherwise.
//
BOOL phidgetCompDigitalOutput_SetValue(PhidgetId id, int iIndex, int iValue)
{
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);
	
	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
		case PHIDCLASS_INTERFACEKIT:
			return CPhidgetInterfaceKit_setOutputState ( ((PhidgetIfKit*) pPhidget)->handle, iIndex, iValue) == 0;
		case PHIDCLASS_RFID:
			return CPhidgetRFID_setOutputState ( ((PhidgetRFID*) pPhidget)->handle, iIndex, iValue) == 0;
			
		default:
			IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompDigitalOutput_SetValue.", phidget_GetDeviceClass(pPhidget)));
			return FALSE;
	}
}

/*++

Routine Description:

	This method sets a semaphore to be signaled when the phidget with the
	corresponding id has changed a components digital output state. 

Arguments:

	id - id of the Phidget to set the semaphore.

	iSemaphoreIndex - the semaphore the iSemaphoreOutputChange is set to.

Return Value:

	TRUE if the operation succeeded, FALSE otherwise.

--*/
BOOL phidgetCompDigitalOutput_SetSemaphoreIndex_StateChange(PhidgetId id, int iSemaphoreIndex)
{
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_INTERFACEKIT:
		((PhidgetIfKit*) pPhidget)->iSemaphoreOutputChange = iSemaphoreIndex;
		IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "New semaphore for <IfKit-OutputChange> is: %d, Phidget id = %i",
			((PhidgetIfKit*) pPhidget)->iSemaphoreOutputChange, pPhidget->id));
		return TRUE;

	case PHIDCLASS_RFID:
		((PhidgetRFID*) pPhidget)->iSemaphoreOutputChange = iSemaphoreIndex;
		IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "New semaphore for <RFID-OutputChange> is: %d, Phidget id = %i",
			((PhidgetRFID*) pPhidget)->iSemaphoreOutputChange, pPhidget->id));
		return TRUE;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetCompDigitalOutput_SetSemaphoreIndex_StateChange.", phidget_GetDeviceClass(pPhidget)));
		return FALSE;
	}
}
