/*++

Copyright (c) 2009
Lysann Kessler, Stephanie Platz, Thomas Klingbeil, Philipp Tessenow, Frank Schlegel;
Michael Haupt, Michael Perscheid, Robert Hirschfeld;
Software Architecture Group, Hasso-Plattner-Institut, University of Potsdam, Germany

This software is published under MIT license.
It uses phidget library by Phidgets Inc. (http://phidgets.com).
See ../../../../license.pdf for more information.

Module Name:

	PhidgetComponent.c

Abstract:

	This file contains Phidget Component general methods common to all components.
	Every phidget has a predefined set of components and an instanciated component
	always belong to a phidget. Only if this phidget is attached the component is
	useable.
	Use phidget_GetDeviceClass(pointerToPhidget) to call the specific phidget
	methods in phidget21 API.

--*/

#include "PhidgetComponent.h"

//////////////////////////////////////////////////////////////////////////
// Phidget Component methods

/*++

Routine Description:

	This method gets a Phidget's Component set change trigger. This mehtod
	should be used if the change trigger is an Integer.
	The Phidget's device class is checked to identify the type of phidget
	and to use the specific get-operation.

Arguments:

	id - id of the Phidget to get the change trigger.

	iIndex - index of the component (may be more than one)

Return Value:

	The current set change trigger of the indexed component (as Integer).

--*/
int phidgetComponent_GetIndexedIntegerChangeTrigger(PhidgetId id, int iIndex)
{
	int iTrigger = 10;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, iTrigger);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
		case PHIDCLASS_INTERFACEKIT:
			CPhidgetInterfaceKit_getSensorChangeTrigger( ((PhidgetIfKit*) pPhidget)->handle, iIndex, &iTrigger);
			break;
			
		default:
			IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetComponent_GetIndexedIntegerChangeTrigger.", phidget_GetDeviceClass(pPhidget)));
	}
	
	return iTrigger;
}

/*++

Routine Description:

	This method sets a Phidget's Component change trigger. This mehtod
	should be used if the change trigger is an Integer.
	The Phidget's device class is checked to identify the type of phidget
	and to use the specific set-operation.

Arguments:

	id - id of the Phidget to set the change trigger.

	iIndex - index of the component (may be more than one)

	iSensitivity - the change trigger value to set to (as Integer)

Return Value:

	TRUE if the operation succeeded, FALSE otherwise.

--*/
BOOL phidgetComponent_SetIndexedIntegerChangeTrigger(PhidgetId id, int iIndex, int iSensitivity)
{
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);
	
	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
		case PHIDCLASS_INTERFACEKIT:
			return CPhidgetInterfaceKit_setSensorChangeTrigger( ((PhidgetIfKit*) pPhidget)->handle, iIndex, iSensitivity) == 0;
			
		default:
			IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetComponent_SetIndexedIntegerChangeTrigger.", phidget_GetDeviceClass(pPhidget)));
			return FALSE;
	}
}

/*++

Routine Description:

	This method gets a Phidget's Component set change trigger. This mehtod
	should be used if the change trigger is a Double.
	The Phidget's device class is checked to identify the type of phidget
	and to use the specific set-operation.

Arguments:

	id - id of the Phidget to get the change trigger.

	iIndex - index of the component (may be more than one)

Return Value:

	The current set change trigger (as Double).

--*/
double phidgetComponent_GetIndexedDoubleChangeTrigger(PhidgetId id, int iIndex)
{
	double dTrigger = 0.1;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, dTrigger);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_ACCELEROMETER:
		CPhidgetAccelerometer_getAccelerationChangeTrigger(((PhidgetAccel*) pPhidget)->handle, iIndex, &dTrigger);
		break;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetComponent_GetIndexedDoubleChangeTrigger.", phidget_GetDeviceClass(pPhidget)));
	}

	return dTrigger;
}

/*++

Routine Description:

	This method sets a Phidget's Component change trigger. This mehtod
	should be used if the change trigger is a Double.
	The Phidget's device class is checked to identify the type of phidget
	and to use the specific set-operation.

Arguments:

	id - id of the Phidget to set the change trigger.

	iIndex - index of the component (may be more than one)

	iSensitivity - the change trigger value to set to (as Double)

Return Value:

	TRUE if the operation succeeded, FALSE otherwise.

--*/
BOOL phidgetComponent_SetIndexedDoubleChangeTrigger(PhidgetId id, int iIndex, double dSensitivity)
{
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);

	//detect the phidget's type
	switch(phidget_GetDeviceClass(pPhidget))
	{
	case PHIDCLASS_ACCELEROMETER:
		return CPhidgetAccelerometer_setAccelerationChangeTrigger(((PhidgetAccel*) pPhidget)->handle, iIndex, dSensitivity) == 0;

	default:
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Wrong Phidget type %i in phidgetComponent_SetIndexedDoubleChangeTrigger.", phidget_GetDeviceClass(pPhidget)));
		return FALSE;
	}
}

