/*++

Copyright (c) 2009
Lysann Kessler, Stephanie Platz, Thomas Klingbeil, Philipp Tessenow, Frank Schlegel;
Michael Haupt, Michael Perscheid, Robert Hirschfeld;
Software Architecture Group, Hasso-Plattner-Institut, University of Potsdam, Germany

This software is published under MIT license.
It uses phidget library by Phidgets Inc. (http://phidgets.com).
See ../../../../license.pdf for more information.

Module Name:

	PhidgetRemote.c

Abstract:

	This file contains Phidget remote methods common to all Phidgets.


--*/

// includes
#include "PhidgetRemote.h"

//////////////////////////////////////////////////////////////////////////
// Phidget methods

/*++

Routine Description:

	This method opens a previously created Phidget with the given serial
	number via remote connection. This is done by providing a Bonjour server
	id and a password.
	See the Phidget Webservice description and Bonjour documentation for
	more information.

Arguments:

	pPhidget - id - ID of the Phidget to open the hardware connection.

	iSerialNumber - The serial number of the Phidget to be opened.
		Every Phidget has an unique serial number. Supply this here to
		open a specific Phidget.
		Set this parameter to -1 to open any Phidget with matching device
		class (supplied when creating the Phidget object).

	pcServerID - String representing the Bonjour server id of the target server.
		Does not need to be zero-terminated.

	iServerIDSize - size of the latter string.

	pcPassword - Password for server connection. Set it to an empty string
		to open the connection without using a password. Does not need to be
		zero-terminated.

	iPasswordSize - size of the latter string, or 0 if no password should
		be used for connection.

Return Value:

	TRUE if the operation succeeded, FALSE otherwise.

Note:

	This function is still under construction and does not work yet!

---TODO---

--*/
BOOL phidget_OpenRemoteID(PhidgetId id, int iSerialNumber, char* pcServerID, int iServerIDSize, char* pcPassword, int iPasswordSize)
{
	BOOL result = FALSE;
	char *pszServerID, *pszPassword;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);

	// do specific open operations for the phidget
	result = phidget_OpenSpecific(pPhidget);
	if(result)
	{
		// convert strings to be zero terminated. The password will be set to NULL, if the string is empty
		pszServerID = phidgetPlugin_ConvertToCString(pcServerID, iServerIDSize);
		pszPassword = iPasswordSize == 0 ? NULL : phidgetPlugin_ConvertToCString(pcPassword, iPasswordSize);
		
		// open the phidget for device connections (Server ID)
		result = phidget_OpenConnection_RemoteID(pPhidget, iSerialNumber, pszServerID, pszPassword);
		
		// free memory allocated for string conversion
		phidgetPlugin_FreeCString(pszServerID);
		phidgetPlugin_FreeCString(pszPassword);
	}

	if(!result)
		IF_DBG(SLog(SLOG_LEVEL_ERROR,
			"Could not open Phidget, id = %i, serial number = %i, server id = %s",
			id, iSerialNumber, pszServerID));
	return result;
}

/*++

Routine Description:

	This method opens a previously created Phidget with the given serial
	number via remote connection. This is done by providing a server address,
	port and password for the target computer.

Arguments:

	id - ID of the Phidget to open the hardware connection.

	iSerialNumber - The serial number of the Phidget to be opened.
		Every Phidget has an unique serial number. Supply this here to
		open a specific Phidget.
		Set this parameter to -1 to open any Phidget with matching device
		class (supplied when creating the Phidget object).
	
	pcServerAddress - String either representing the target server name or an
		IP address of the server. Does not need to be zero-terminated.

	iServerAddressSize - size of the latter string.

	iPort - Port number at which to connect to the Phidget Web Service.

	pcPassword - Password for server connection. Set it to an empty string
		to open the connection without using a password. Does not need to be
		zero-terminated.

	iPasswordSize - size of the latter string, or 0 if no password should
		be used for connection.

Return Value:

	TRUE if the operation succeeded, FALSE otherwise.

--*/
BOOL phidget_OpenRemoteIP(PhidgetId id, int iSerialNumber, char* pcServerAddress, int iServerAddressSize, int iPort, char* pcPassword, int iPasswordSize)
{
	BOOL result = FALSE;
	char *pszServerAddress, *pszPassword;
	Phidget* pPhidget = phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, FALSE);

	// do specific open operations for the phidget
	result = phidget_OpenSpecific(pPhidget);
	if(result)
	{
		// convert strings to be zero terminated. The password will be set to NULL, if the string is empty
		pszServerAddress = phidgetPlugin_ConvertToCString(pcServerAddress, iServerAddressSize);
		pszPassword = iPasswordSize == 0 ? NULL : phidgetPlugin_ConvertToCString(pcPassword, iPasswordSize);

		// open the phidget for device connections (IP)
		result = phidget_OpenConnection_RemoteIP(pPhidget, iSerialNumber, pszServerAddress, iPort, pszPassword);

		// free memory allocated for string conversion
		phidgetPlugin_FreeCString(pszServerAddress);
		phidgetPlugin_FreeCString(pszPassword);
	}

	if(!result)
		IF_DBG(SLog(SLOG_LEVEL_ERROR,
			"Could not open Phidget, id = %i, serial number = %i, server address = %s, port = %i",
			id, iSerialNumber, pszServerAddress, iPort));
	return result;
}

/*++

Routine Description:

	This method is a helper function for phidget_OpenRemoteID. It actually
	establishes the hardware connection to the remote phidget.

Arguments:

	pPhidget - pointer of the Phidget to open the hardware connection.

	iSerialNumber - The serial number of the Phidget to be opened.
		Every Phidget has an unique serial number. Supply this here to
		open a specific Phidget.
		Set this parameter to -1 to open any Phidget with matching device
		class (supplied when creating the Phidget object).

	pszServerID - Zero-terminated String representing the Bonjour server id
		of the target server.

	pszPassword - Zero-terminated String to use as password for server
		connection. Or: NULL to open the connection without using a
		password.

Return Value:

	TRUE if the operation succeeded, FALSE otherwise.

Note:

	This function is still under construction and does not work yet!

---TODO---

--*/
BOOL phidget_OpenConnection_RemoteID(Phidget* pPhidget, int iSerialNumber, char* pszServerID, char* pszPassword)
{
	BOOL result;

	// open the connection
	result = CPhidget_openRemote(pPhidget->handle, iSerialNumber, pszServerID, pszPassword) == 0;

	// log
	if(result)
		IF_DBG(SLog(SLOG_LEVEL_INFO, "Phidget opened remotely, serial number = %i, ServerID = %s\n%s",
		iSerialNumber, pszServerID, phidgetPlugin_GetPhidgetDescription(pPhidget)));
	else
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Could not open remotely, serial number = %i, PhidgetId = %i, ServerID = %s\n",
		iSerialNumber, pPhidget->id, pszServerID));

	return result;
}

/*++

Routine Description:

	This method is a helper function for phidget_OpenRemoteIP. It actually
	establishes the hardware connection to the remote phidget.

Arguments:

	pPhidget - pointer of the Phidget to open the hardware connection.

	iSerialNumber - The serial number of the Phidget to be opened.
		Every Phidget has an unique serial number. Supply this here to
		open a specific Phidget.
		Set this parameter to -1 to open any Phidget with matching device
		class (supplied when creating the Phidget object).

	pszServerAddress - Zero-terminated String either representing the target
		server name or an IP address of the server.

	iPort - Port number at which to connect to the Phidget Web Service.

	pszPassword - Zero-terminated String to use as password for server
		connection. Or: NULL to open the connection without using a
		password.

Return Value:

	TRUE if the operation succeeded, FALSE otherwise.

--*/
BOOL phidget_OpenConnection_RemoteIP(Phidget* pPhidget, int iSerialNumber, char* pszServerAddress, int iPort, char* pszPassword)
{
	BOOL result;

	// open the connection
	result = CPhidget_openRemoteIP(pPhidget->handle, iSerialNumber, pszServerAddress, iPort, pszPassword) == 0;

	// log
	if(result)
		IF_DBG(SLog(SLOG_LEVEL_INFO, "Phidget opened remotely, serial number = %i, ServerAddress = %s, Port = %i\n%s",
		iSerialNumber, pszServerAddress, iPort, phidgetPlugin_GetPhidgetDescription(pPhidget)));
	else
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Could not open remotely, serial number = %i, PhidgetId = %i, ServerAddress = %s, Port = %i\n",
		iSerialNumber, pPhidget->id, pszServerAddress, iPort));

	return result;
}
