/*++

Copyright (c) 2009
Lysann Kessler, Stephanie Platz, Thomas Klingbeil, Philipp Tessenow, Frank Schlegel;
Michael Haupt, Michael Perscheid, Robert Hirschfeld;
Software Architecture Group, Hasso-Plattner-Institut, University of Potsdam, Germany

This software is published under MIT license.
It uses phidget library by Phidgets Inc. (http://phidgets.com).
See ../../../../license.pdf for more information.

Module Name:

	PhidgetServo.c

Abstract:

	This file contains Phidget Servo specific methods. A Phidget Servo
	has a component servo motor. Both modules - PhidgetCompServoMotor.c
	and PhidgetServo.c - implement all methods supplied by the phidget21
	API for an Servo.

--*/

#include "PhidgetServo.h"

//////////////////////////////////////////////////////////////////////////
// forward declarations: callback functions

int CALLBACK phidgetServo_positionChangeHandler(CPhidgetServoHandle servoHandle, void *pUserPtr, int iIndex, double fPosition);

//////////////////////////////////////////////////////////////////////////
// PhidgetServo methods

/*++

Routine Description:

	This method creates an PhidgetServo struct and allocates memory.
	To identify the phidgets type the deviceClass is set here.
	Additionally do common creational settings.

Arguments:

	None.

Return Value:

	Squeak VM identifier for the PhigdetId Object.

--*/
sqInt phidgetServo_Create()
{
	// allocate memory for a PhidgetServo structure
	PhidgetServo* pPhidget = calloc(1, sizeof(PhidgetServo));

	// initialise
	pPhidget->deviceClass = PHIDCLASS_SERVO;
	phidgetServo_ResetSemaphores(pPhidget);

	// get id and add to global list. return the id
	return phidget_Create((Phidget*)pPhidget);
}

//
//reset semaphores
//
void phidgetServo_ResetSemaphores(PhidgetServo *pPhidget)
{
	if(pPhidget != NULL)
	{
		phidget_ResetSemaphores((Phidget*)pPhidget);
		pPhidget->iSemaphorePositionChange = 0;
	}
	else
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Null pointer given in phidgetServo_ResetSemaphores for pPhidget."));
}


/*++

Routine Description:

	This method is called while opening a phidget. The actuall specific
	phidget object will be created and the specific callbacks will be
	registered.

Arguments:

	pPhidget - pointer of the Phidget to open the hardware connection.

Return Value:

	TRUE if the operation succeeded, FALSE otherwise.

--*/
BOOL phidgetServo_Open(PhidgetServo *pPhidget)
{
	if(pPhidget == NULL)
	{
		IF_DBG(SLog(SLOG_LEVEL_ERROR, "Null pointer given in phidgetServo_Open for pPhidget."));
		return FALSE;
	}

	//create the Servo object
	pPhidget->handle = 0;
	CPhidgetServo_create(&(pPhidget->handle));

	//Registers the callbacks that will run if a position changes.
	//Requires the handle for the Phidget, the function that will be called, and an arbitrary pointer that will be supplied to the callback function (may be NULL).
	CPhidgetServo_set_OnPositionChange_Handler (pPhidget->handle, phidgetServo_positionChangeHandler, (void *)pPhidget->id);

	// do phidget common open operations
	phidget_Open_Base((Phidget *)pPhidget);
	return TRUE;
}

//////////////////////////////////////////////////////////////////////////
// callback functions

/*++

Routine Description:

	This callback is called when a motor of a Phidget Servo with the id 
	pUserPtr has changed its position.
	It signals the semaphore iSemaphorePositionChange.

Arguments:

	servoHandle - handle of the Servo to signal the semaphore.

	pUserPtr - Callback function pointer.

	iIndex - index of the motor that generated the event

	fPosition - the motor read position

Return Value:

	1 if the operation succeeded, 0 otherwise.

--*/
int CALLBACK phidgetServo_positionChangeHandler(CPhidgetServoHandle servoHandle, void *pUserPtr, int iIndex, double fPosition)
{
	PhidgetId id = (PhidgetId) pUserPtr;
	PhidgetServo* pPhidget = (PhidgetServo*) phidgetPlugin_GetPhidgetFromId_ElseReturn(id, pPhidget, 1);

	IF_DBG(SLog(SLOG_LEVEL_VERBOSE, "<Servo-PositionChange> id = %i, semaphore = %i, index = %i, value = %f",
		pPhidget->id, pPhidget->iSemaphorePositionChange, iIndex, fPosition));
	
	if (pPhidget->iSemaphorePositionChange > 0)
		interpreterProxy->signalSemaphoreWithIndex(pPhidget->iSemaphorePositionChange);
	else
		IF_DBG(SLog(SLOG_LEVEL_WARNING, "No semaphore set for <Servo-PositionChange>, id = %i", pPhidget->id));
	return 0;
}
